@@ -17,13 +17,15 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "call-window.h"
-#include "ui_call-window.h"
 
 #include "status-area.h"
 #include "dtmf-handler.h"
+#include "dtmf-qml.h"
 #include "../libktpcall/call-channel-handler.h"
 
-#include <QtGui/QCloseEvent>
+#include <QCloseEvent>
+#include <QVBoxLayout>
+#include <QGraphicsObject>
 
 #include <TelepathyQt/ReferencedHandles>
 #include <TelepathyQt/AvatarData>
@@ -36,7 +38,12 @@
 #include <KToolBar>
 #include <KMessageWidget>
 
+#include <QDeclarativeView>
+#include <QDeclarativeContext>
+#include <QDeclarativeEngine>
 #include <QGst/ElementFactory>
+#include <QGst/Ui/GraphicsVideoSurface>
+#include <QGst/Init>
 
 struct CallWindow::Private
 {
@@ -49,15 +56,21 @@ struct CallWindow::Private
     Tp::CallChannelPtr callChannel;
     CallChannelHandler *channelHandler;
     StatusArea *statusArea;
-    Ui::CallWindow ui;
     bool callEnded;
 
+    QmlInterface *qmlUi;
+    KMessageWidget *errorWidget;
+    DtmfQml *dtmfQml;
+
     KToggleAction *showMyVideoAction;
     KToggleAction *showDtmfAction;
     KToggleAction *sendVideoAction;
     KToggleAction *muteAction;
     KAction *holdAction;
     KAction *hangupAction;
+    KAction *goToSystemTrayAction;
+    KAction *restoreAction;
+    KToggleAction *fullScreenAction;
 
     VideoDisplayFlags currentVideoDisplayState;
     VideoContentHandler *videoContentHandler;
@@ -71,32 +84,29 @@ CallWindow::CallWindow(const Tp::CallChannelPtr & callChannel)
     : KXmlGuiWindow(), d(new Private)
 {
     d->callChannel = callChannel;
+    setupActions();
 
     //create ui
-    d->ui.setupUi(this);
+    setupQmlUi();
     d->statusArea = new StatusArea(statusBar());
-    d->ui.errorWidget->hide();
-    d->ui.errorWidget->setMessageType(KMessageWidget::Error);
-    setupActions();
-    setupGUI(QSize(428, 395), ToolBar | Keys | StatusBar | Create, QLatin1String("callwindowui.rc"));
+
+    d->errorWidget->hide();
+    d->errorWidget->setMessageType(KMessageWidget::Error);
+    setupGUI(QSize(750, 450), ToolBar | Keys | StatusBar | Create, QLatin1String("callwindowui.rc"));
     setAutoSaveSettings(QLatin1String("CallWindow"), false);
     toolBar()->setToolButtonStyle(Qt::ToolButtonIconOnly);
 
     DtmfHandler *handler = new DtmfHandler(d->callChannel, this);
-    handler->connectDtmfWidget(d->ui.dtmfWidget);
+    d->dtmfQml = new DtmfQml(this);
+    handler->connectDtmfQml(d->dtmfQml);
 
     //TODO handle member joining later
     if (!d->callChannel->remoteMembers().isEmpty()) {
         Tp::ContactPtr remoteMember = *d->callChannel->remoteMembers().begin();
-        d->ui.remoteUserDisplayLabel->setText(QString::fromAscii(
-                "<html><body>"
-                "<p align=\"center\"><img src=\"%1\" /></p>"
-                "<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px;\">"
-                    "<span style=\" font-weight:600;\">%2</span></p>"
-                "</body></html>")
-                .arg(remoteMember->avatarData().fileName, remoteMember->alias()));
+        d->qmlUi->setLabel(i18n("Call with %1", remoteMember->alias()), remoteMember->avatarData().fileName );
         setWindowTitle(i18n("Call with %1", remoteMember->alias()));
     }
+    setupSystemTray();
 }
 
 CallWindow::~CallWindow()
@@ -124,6 +134,7 @@ void CallWindow::setStatus(Status status, const Tp::CallStateReason & reason)
         d->statusArea->startDurationTimer();
         if (d->callChannel.data()->hasInterface(TP_QT_IFACE_CHANNEL_INTERFACE_HOLD)) {
             d->holdAction->setEnabled(true);
+            d->qmlUi->setHoldEnabled(true);
             connect(d->callChannel.data(), SIGNAL(localHoldStateChanged(Tp::LocalHoldState,Tp::LocalHoldStateReason)),
                     SLOT(onHoldStatusChanged(Tp::LocalHoldState,Tp::LocalHoldStateReason)));
         }
@@ -210,6 +221,7 @@ void CallWindow::setStatus(Status status, const Tp::CallStateReason & reason)
         break;
       }
       d->holdAction->setEnabled(false);
+      d->qmlUi->setHoldEnabled(false);
     default:
         Q_ASSERT(false);
     }
@@ -229,6 +241,8 @@ void CallWindow::onContentAdded(CallContentHandler *contentHandler)
         d->muteAction->setEnabled(vol->volumeControlSupported());
         connect(vol, SIGNAL(volumeControlSupportedChanged(bool)),
                 d->muteAction, SLOT(setEnabled(bool)));
+        connect(vol, SIGNAL(volumeControlSupportedChanged(bool)),
+                d->qmlUi, SLOT(setSoundEnabled(bool)));
         d->muteAction->setChecked(vol->isMuted());
         connect(vol, SIGNAL(mutedChanged(bool)), d->muteAction, SLOT(setChecked(bool)));
         d->muteAction->setProperty("volumeController", QVariant::fromValue<QObject*>(vol));
@@ -249,6 +263,9 @@ void CallWindow::onContentAdded(CallContentHandler *contentHandler)
                 SLOT(onRemoteVideoSendingStateChanged(Tp::ContactPtr,bool)));
 
         d->statusArea->showVideoStatusIcon(true);
+        d->showMyVideoAction->setEnabled(true);
+        // would there be a good way of saving the users preference between runs?
+        d->showMyVideoAction->setChecked(true);
     }
 }
 
@@ -308,72 +325,34 @@ void CallWindow::onRemoteVideoSendingStateChanged(const Tp::ContactPtr & contact
     }
 }
 
-QGst::ElementPtr CallWindow::tryVideoSink(const char *videoSink)
-{
-    QGst::ElementPtr sink = QGst::ElementFactory::make(videoSink);
-    if (!sink) {
-        kDebug() << "Could not make video sink" << videoSink;
-        return sink;
-     }
-
-    if (!sink->setState(QGst::StateReady)) {
-        kDebug() << "Video sink" << videoSink << "does not want to become ready";
-        return QGst::ElementPtr();
-    }
-
-    kDebug() << "Using video sink" << videoSink;
-    sink->setState(QGst::StateNull);
-    return sink;
-}
-
-QGst::ElementPtr CallWindow::constructVideoSink()
-{
-    QGst::ElementPtr sink = tryVideoSink("xvimagesink");
-    if (!sink) {
-        sink = tryVideoSink("ximagesink");
-    }
-
-    sink->setProperty("force-aspect-ratio", true);
-
-    return sink;
-
-}
 void CallWindow::changeVideoDisplayState(VideoDisplayFlags newState)
 {
     VideoDisplayFlags oldState = d->currentVideoDisplayState;
 
     if (oldState.testFlag(LocalVideoPreview) && !newState.testFlag(LocalVideoPreview)) {
         d->videoContentHandler->unlinkVideoPreviewSink();
-        d->ui.videoPreviewWidget->setVideoSink(QGst::ElementPtr());
     } else if (!oldState.testFlag(LocalVideoPreview) && newState.testFlag(LocalVideoPreview)) {
-        QGst::ElementPtr localVideoSink = constructVideoSink();
+        QGst::ElementPtr localVideoSink = d->qmlUi->getVideoPreviewSink();
         if (localVideoSink) {
-            d->ui.videoPreviewWidget->setVideoSink(localVideoSink);
-            d->videoContentHandler->linkVideoPreviewSink(localVideoSink);
+            d->videoContentHandler->linkVideoPreviewSink(localVideoSink );
         }
     }
 
     if (oldState.testFlag(RemoteVideo) && !newState.testFlag(RemoteVideo)) {
         d->videoContentHandler->unlinkRemoteMemberVideoSink(d->remoteVideoContact);
-        d->ui.videoWidget->setVideoSink(QGst::ElementPtr());
     } else if (!oldState.testFlag(RemoteVideo) && newState.testFlag(RemoteVideo)) {
-        QGst::ElementPtr remoteVideoSink = constructVideoSink();
+        QGst::ElementPtr remoteVideoSink = d->qmlUi->getVideoSink();
         if (remoteVideoSink) {
-            d->ui.videoWidget->setVideoSink(remoteVideoSink);
             d->videoContentHandler->linkRemoteMemberVideoSink(d->remoteVideoContact, remoteVideoSink);
         }
     }
 
     if (newState == NoVideo) {
-        d->ui.callStackedWidget->setCurrentIndex(0);
+        //d->ui.callStackedWidget->setCurrentIndex(0);
+        d->qmlUi->setShowVideo(false);
     } else {
-        d->ui.callStackedWidget->setCurrentIndex(1);
-
-        if (newState.testFlag(LocalVideoPreview)) {
-            d->ui.videoPreviewWidget->show();
-        } else {
-            d->ui.videoPreviewWidget->hide();
-        }
+        //d->ui.callStackedWidget->setCurrentIndex(1);
+        d->qmlUi->setShowVideo(true);
     }
 
     d->currentVideoDisplayState = newState;
@@ -381,18 +360,26 @@ void CallWindow::changeVideoDisplayState(VideoDisplayFlags newState)
 
 void CallWindow::setupActions()
 {
-     //TODO implement this feature
     d->showMyVideoAction = new KToggleAction(i18nc("@action", "Show my video"), this);
     d->showMyVideoAction->setIcon(KIcon("camera-web"));
     d->showMyVideoAction->setEnabled(false);
+    d->showMyVideoAction->setChecked(false);
     actionCollection()->addAction("showMyVideo", d->showMyVideoAction);
 
     d->showDtmfAction = new KToggleAction(i18nc("@action", "Show dialpad"), this);
     d->showDtmfAction->setIcon(KIcon("phone"));
     d->showDtmfAction->setEnabled(false);
-    connect(d->showDtmfAction, SIGNAL(toggled(bool)), SLOT(toggleDtmf(bool)));
     actionCollection()->addAction("showDtmf", d->showDtmfAction);
 
+    d->goToSystemTrayAction = new KAction(i18nc("@action", "Hide window"), this);
+    d->goToSystemTrayAction->setEnabled(true);
+    connect(d->goToSystemTrayAction, SIGNAL(triggered(bool)), this, SLOT(hide()));
+    actionCollection()->addAction("goToSystemTray", d->goToSystemTrayAction);
+
+    d->restoreAction= new KAction(i18nc("@action", "Restore window"), this);
+    d->restoreAction->setEnabled(true);
+    connect(d->restoreAction, SIGNAL(triggered(bool)), this, SLOT(show()));
+
     //TODO implement this feature
     d->sendVideoAction = new KToggleAction(i18nc("@action", "Send video"), this);
     d->sendVideoAction->setIcon(KIcon("webcamsend"));
@@ -414,6 +401,11 @@ void CallWindow::setupActions()
     d->hangupAction = new KAction(KIcon("call-stop"), i18nc("@action", "Hangup"), this);
     connect(d->hangupAction, SIGNAL(triggered()), SLOT(hangup()));
     actionCollection()->addAction("hangup", d->hangupAction);
+
+    d->fullScreenAction = new KToggleAction(KIcon("view-fullscreen"),i18nc("@action", "Full Screen"), this);
+    d->fullScreenAction->setEnabled(true);
+    connect(d->fullScreenAction, SIGNAL(triggered()), SLOT(toggleFullScreen()));
+    actionCollection()->addAction("fullScreen", d->fullScreenAction);
 }
 
 void CallWindow::checkEnableDtmf()
@@ -436,7 +428,12 @@ void CallWindow::checkEnableDtmf()
 
 void CallWindow::toggleDtmf(bool checked)
 {
-    d->ui.dtmfStackedWidget->setCurrentIndex(checked ? 1 : 0);
+    if (checked) {
+        exitFullScreen();
+        d->dtmfQml->show();
+    } else {
+        d->dtmfQml->hide();
+    }
 }
 
 void CallWindow::toggleMute(bool checked)
@@ -459,6 +456,7 @@ void CallWindow::hangup()
 
 void CallWindow::closeEvent(QCloseEvent *event)
 {
+    systemtrayicon->setActivateNext(false);
     if (!d->callEnded) {
         kDebug() << "Ignoring close event";
         hangup();
@@ -487,8 +485,8 @@ void CallWindow::hold()
 void CallWindow::holdOperationFinished(Tp::PendingOperation* operation)
 {
     if (operation->isError()) {
-        d->ui.errorWidget->setText(i18nc("@info:error", "There was an error while pausing the call"));
-        d->ui.errorWidget->animatedShow();
+        d->errorWidget->setText(i18nc("@info:error", "There was an error while pausing the call"));
+        d->errorWidget->animatedShow();
         return;
     }
 }
@@ -507,7 +505,9 @@ void CallWindow::onHoldStatusChanged(Tp::LocalHoldState state, Tp::LocalHoldStat
             d->statusArea->setMessage(StatusArea::Error, i18nc("@info:error", "Unknown error"));
         }
         d->holdAction->setEnabled(true);
+        d->qmlUi->setHoldEnabled(true);
         d->holdAction->setIcon(KIcon("media-playback-start"));
+        d->qmlUi->setChangeHoldIcon("start");
         d->statusArea->stopDurationTimer();
         break;
 
@@ -519,21 +519,134 @@ void CallWindow::onHoldStatusChanged(Tp::LocalHoldState state, Tp::LocalHoldStat
             d->statusArea->setMessage(StatusArea::Error, i18nc("@info:error", "Unknown error"));
         }
         d->holdAction->setEnabled(true);
+        d->qmlUi->setHoldEnabled(true);
         d->holdAction->setIcon(KIcon("media-playback-pause"));
+        d->qmlUi->setChangeHoldIcon("pause");
         d->statusArea->startDurationTimer();
         break;
 
     case Tp::LocalHoldStatePendingHold:
         d->statusArea->setMessage(StatusArea::Status, i18nc("@info:status", "Putting the call on hold..."));
         d->holdAction->setEnabled(false);
+        d->qmlUi->setHoldEnabled(false);
         break;
 
     case Tp::LocalHoldStatePendingUnhold:
         d->statusArea->setMessage(StatusArea::Status, i18nc("@info:status", "Resuming the call..."));
         d->holdAction->setEnabled(false);
+        d->qmlUi->setHoldEnabled(false);
         break;
 
     default:
         d->statusArea->setMessage(StatusArea::Error, i18nc("@info:error", "Internal Error"));
     }
 }
+
+void CallWindow::setupSystemTray()
+{
+    KMenu *trayIconMenu=new KMenu(this);
+    systemtrayicon=new SystemTrayIcon(this);
+
+    //Save the title
+    trayIconMenu->setTitle(windowTitle());
+
+    //Show the title
+    trayIconMenu->addAction(windowTitle());
+    trayIconMenu->addSeparator();
+
+    //Actions
+    trayIconMenu->addAction(d->hangupAction);
+    trayIconMenu->addAction(d->holdAction);
+    trayIconMenu->addAction(d->sendVideoAction);
+    trayIconMenu->addAction(d->muteAction);
+    trayIconMenu->addAction(d->restoreAction);
+    trayIconMenu->addAction(KStandardAction::close(this, SLOT(close()), actionCollection()));
+
+    systemtrayicon->setAssociatedWidget(this);
+
+    //Restore when left click
+    connect(systemtrayicon, SIGNAL(activateRequested(bool,QPoint)), this, SLOT(show()));
+    systemtrayicon->setToolTip("call-start", windowTitle(),"");
+
+    systemtrayicon->setContextMenu(trayIconMenu);
+}
+
+void CallWindow::showEvent(QShowEvent* event)
+{
+    KXmlGuiWindow::showEvent(event);
+    systemtrayicon->setStatus(KStatusNotifierItem::Passive);
+}
+
+void CallWindow::hideEvent(QHideEvent* event)
+{
+    if (isHidden()) {
+        systemtrayicon->show();
+    }
+    KXmlGuiWindow::hideEvent(event);
+}
+
+
+/*! Creates a QmlInterface and sets it as central widget.
+ * \a Ekaitz.
+ */
+void CallWindow::setupQmlUi()
+{
+    QVBoxLayout *layout = new QVBoxLayout(this);
+    QWidget *centralWidget = new QWidget(this);
+    d->qmlUi = new QmlInterface( this );
+    d->errorWidget = new KMessageWidget( this );
+
+    layout->addWidget(d->errorWidget);
+    layout->addWidget(d->qmlUi);
+
+    centralWidget->setLayout(layout);
+    setCentralWidget(centralWidget);
+
+    // grab root object so we can access QML signals.
+    QObject *root = d->qmlUi->rootObject();
+
+    //QML-UI <---> Actions
+    //Show dialpad
+    connect(root, SIGNAL(showDialpadClicked(bool)), SLOT(toggleDtmf(bool)));
+    connect(d->showDtmfAction, SIGNAL(toggled(bool)), d->qmlUi, SIGNAL(showDialpadChangeState(bool)));
+    connect(root, SIGNAL(showDialpadClicked(bool)), d->showDtmfAction, SLOT(setChecked(bool)));
+    //Mute <-> Sound activated
+    connect(root,SIGNAL(muteClicked(bool)), SLOT(toggleMute(bool)));
+    connect(d->muteAction, SIGNAL(toggled(bool)), d->qmlUi, SIGNAL(soundChangeState(bool)));
+    connect(root,SIGNAL(muteClicked(bool)), d->muteAction, SLOT(setChecked(bool)));
+    //Hold
+    connect(root,SIGNAL(holdClicked()),SLOT(hold()));
+    //Hangup
+    connect(root,SIGNAL(hangupClicked()),SLOT(hangup()));
+    //Exit FullScreen
+    connect(root,SIGNAL(exitFullScreen()),SLOT(exitFullScreen()));
+}
+
+/*!This function makes the central QML widget go to full screen. To exit fullScreen mode, press \a Esc.
+ * \a Ekaitz.
+ * \sa QmlInterface::exitFullScreen(), hideWithSystemTray(), showWithSystemTray().
+ */
+void CallWindow::toggleFullScreen()
+{
+    if (d->qmlUi->isFullScreen()) {
+        d->fullScreenAction->setChecked(false);
+        d->qmlUi->setWindowFlags(Qt::Widget);
+        setCentralWidget(d->qmlUi);
+        d->qmlUi->showNormal();
+        show();
+    } else {
+        d->fullScreenAction->setChecked(true);
+        d->qmlUi->setWindowFlags(Qt::Window);
+        d->qmlUi->showFullScreen();
+        hide();
+    }
+}
+
+void CallWindow::exitFullScreen()
+{
+    if (d->qmlUi->isFullScreen()) {
+        // avoid duplicating isFullScreen true branch.
+        toggleFullScreen();
+    }
+}
+
